!===================================================
!
! SUBROUTINE: proteusread.F
!
! PURPOSE   : reads proteus dump
!
! INPUT     : incbdy  : 0: dump active grid, 1: dump full grid
!             dmpcount: number of dump. Will be increased by 1 upon return.
!
!===================================================

subroutine proteusread(nx      , ny      , nz     ,         &
                       iwhich  , dmpcount, timec  , iter   ,&
                       field   ,                            &
                       dmpunit , logunit , idebug) 

  implicit none

  integer nx       , nxtot  , ixbeg , ixend, tnx, &
          ny       , nytot  , iybeg , iyend, tny, &
          nz       , nztot  , izbeg , izend, tnz, &
          ndim     , ido    , nvardump
  integer dmpcount , iter   , incbdy, iwhich , tdmpcount
  integer dmpunit  , logunit, idebug, ierr   , nvar   , naxis

  double precision timec, gamma

  double precision field(1:nx,1:ny,1:nz)

  parameter(ido = 1)

  character(12) fnmfield
 
!----------------------------------------------------

  if (iwhich .eq.  4) write(fnmfield,'(a4,i4.4,a4)') &
    "dens",dmpcount,".f7u"
  if (iwhich .eq.  5) write(fnmfield,'(a4,i4.4,a4)') &
    "prss",dmpcount,".f7u"
  if (iwhich .eq.  6) write(fnmfield,'(a4,i4.4,a4)') &
    "momx",dmpcount,".f7u"
  if (iwhich .eq.  7) write(fnmfield,'(a4,i4.4,a4)') &
    "momy",dmpcount,".f7u"
  if (iwhich .eq.  8) write(fnmfield,'(a4,i4.4,a4)') &
    "momz",dmpcount,".f7u"
  if (iwhich .eq.  9) write(fnmfield,'(a4,i4.4,a4)') &
    "magx",dmpcount,".f7u"
  if (iwhich .eq. 10) write(fnmfield,'(a4,i4.4,a4)') &
    "magy",dmpcount,".f7u"
  if (iwhich .eq. 11) write(fnmfield,'(a4,i4.4,a4)') &
    "magz",dmpcount,".f7u"
  if (iwhich .eq. 21) write(fnmfield,'(a4,i4.4,a4)') &
    "deh2",dmpcount,".f7u"

  if (idebug .ge. 1) then
    write(logunit,'("PROTEUSREAD: reading file ",a12)') fnmfield
  endif  

  open(unit=dmpunit,file=fnmfield,form="unformatted",status="unknown")
  read(dmpunit) ndim, incbdy
  read(dmpunit) tnx, nxtot, ixbeg, ixend
  read(dmpunit) tny, nytot, iybeg, iyend
  read(dmpunit) tnz, nztot, izbeg, izend
  read(dmpunit) tdmpcount, timec, iter
  read(dmpunit) field
  close(dmpunit)

!===================================================

  return
end

