;;========================================
;;
;; coretree.pro
;;
;; DESCRIPTION:
;;
;;      Modification of tree_example.pro, containing comparison
;;      routine etc for the core finding algorithm.
;;
;;============================================================================

pro cmpindi, d1, d2, result
;;
;; compare i-indices

  result= (d1.indi lt d2.indi)*(-1) + (d1.indi eq d2.indi)*0  $
			+ (d1.indi gt d2.indi)*1
end
;;============================================================================

pro cmpindj, d1, d2, result
;;
;; compare j-indices

  result= (d1.indj lt d2.indj)*(-1) + (d1.indj eq d2.indj)*0  $
			+ (d1.indj gt d2.indj)*1
end
;;============================================================================

pro cmpindk, d1, d2, result
;;
;; compare k-indices

  result= (d1.indk lt d2.indk)*(-1) + (d1.indk eq d2.indk)*0  $
			+ (d1.indk gt d2.indk)*1
end
;;============================================================================

pro cmprho, d1, d2, result
;;
;; compare densities
  result= (d1.rho lt d2.rho)*(-1) + (d1.rho eq d2.rho)*0  $
                        + (d1.rho gt d2.rho)*1
end

;;============================================================================

pro cmpthreshold, d1, rhothresh, result
;;
;; compare densities
  result= (d1.rho lt rhothresh)*(-1) + (d1.rho eq rhothresh)*0  $
                        + (d1.rho gt rhothresh)*1
end

;;============================================================================

pro isintree, d1, mask, result
;;
;; compare densities. returns 0, if d1 is not in tree.
  result= (mask[d1.indi,d1.indj,d1.indk] eq 1)
end

;;============================================================================

pro setmask, d1, mask
;;
;; sets flag in mask to denote cell belonging to core
  mask[d1.indi,d1.indj,d1.indk] = 1
end

;;============================================================================

pro assigndata, d1, rho, i, j, k
;;
;; sets flag in mask to denote cell belonging to core
  d1.indi = i
  d1.indj = j
  d1.indk = k
  d1.rho  = rho[i,j,k]
end

;;============================================================================

pro printstruct, data
;;
;; Print the data

   print,format='("PRINTSTRUCT: (",i4,",",i4,",",i4,"): rho = ",e13.5)',$
         data.indi,data.indj,data.indk,data.rho

end

;;===========================================================================

pro coretree

; main program for core finding.
; 

  mdl       = 'aa'
  dump      = 5
  icore     = 2
  nx        = 51l
  ny        = 51l
  nz        = 51l

;---------------------------------------

  rho = fltarr(nx,ny,nz)

  nodedata = {indi:-1l, indj:-1l, indk:-1l, rho:0.0} 

; read the subcore file
  imax = 1l
  jmax = 1l
  kmax = 1l
  timec= 1.0
  ticore = 0
  fnm = disk+'/'+'subcore'+string(icore,format='(i2.2)')+'rho'+string(dump,format='(i3.3)')+mdl+'.dmp'
  print,'CORETREE: reading file ',fnm
  readcube,fnm,rho,nx,ny,nz,ticore,imax,jmax,kmax,timec,/swap

  print,format='("CORETREE: time = ",e13.5,", max(rho) = ",e13.5," at (",i4,",",i4,",",i4,")")',$
               timec,rho[imax,jmax,kmax],imax,jmax,kmax

  rhothresh = median(rho)
  print,format='("CORETREE: rhothresh = ",e13.5)',rhothresh
  
  tree_new, core_tree, 'cmp  


   MyData = replicate(DATATYPE, Cnt)

;; Create some data

   MyData.data = randomn(Seed, Cnt)

;; Put some time tags on the data

   MyData.time = Systime(1)*randomu(Seed, Cnt)

;; Now lets make a tree that will store the data in time order and one
;; that holds the data in data order.  "Tree_new" is found in the file
;; idl_tree.pro:

   Tree_new, DataTree, 'CmpData'
   Tree_new, TimeTree, 'CmpTime'

;; Go through and place the data into the trees:

   for i = 0, Cnt-1 do begin
      Tree_Insert, TimeTree, MyData(i)
      Tree_Insert, DataTree, MyData(i)
   endfor

;; Print out the trees

   Print, "Here is the tree sorted by TIME values:"
   Tree_Traverse, TimeTree,  'PrintStruct', /INORDER

   Print, "Here is the tree sorted by DATA values:", Format='(/,A)'
   Tree_Traverse, DataTree, 'PrintStruct', /INORDER

;;
   Print, "Press any key to continue...", Format='(/,A,$)'
   a=Get_Kbrd(1)
;
;; Lets search for some data
;
   Print, ""
;
;;Search for the fourth data value:
;
   Print, "Searching for the fourth data value:", MyData(3)
;
   pData = Tree_Search(TimeTree, MyData(3))
;
   if(not PTR_VALID(pData))then 	$
	print,"Data Not Found"				$
   else BEGIN
	print, "Data Found, Value :", *pData
   ENDelse
;
;; Now delete some nodes in the tree
;
   Print,""
   Print, "Deleting the node that contains:", MyData(3)

   Tree_DeleteNode, DataTree, MyData(3)
   Tree_DeleteNode, TimeTree, MyData(3)
;
;; Now do another search for the node we just deleted
;
   Print, ""
;
   Print, "Now performing a search for the node we just deleted..."
   Print, "Searching for :", MyData(3)
;
   pData = Tree_Search(TimeTree, MyData(3))
;
   if(not PTR_VALID(pData(0)))then   $
        print,"Data Not Found"                          $
   else BEGIN
        print, "Data Found, Value :", *pData
   ENDelse
;
   Print,""
;
   Print, "Now we also delete the node containing the second data value."
   Print, "Deleting node :", MyData(1)
;
   Tree_DeleteNode, DataTree, MyData(1)
   Tree_DeleteNode, TimeTree, MyData(1)
;
   Print, ""
   Print, "Finally, we print the resulting trees..."
   Print, "Sorted by TIME values:", Format='(/,A)'
   Tree_Traverse, TimeTree, 'printStruct', /INORDER
;
   Print, "Sorted by DATA values:", Format='(/,A)'
   Tree_Traverse, DataTree, 'printStruct', /INORDER
;
;; Now delete the trees
;
   Tree_Delete, DataTree
   Tree_Delete, TimeTree
;
END
