!=========================================
!
! coretree.F
!
! PURPOSE: finds core by building a tree, starting
!          at density maximum.
!
!=========================================

program coretree

  use treelib ! module treelib.F, containing tree-functions

  implicit none

  integer nx, ny, nz, imax, jmax, kmax, i, j, k
  integer logunit, dmpunit, inpunit, idebug, count
  integer dmp, icore
  integer maxlevel, icrit

  logical acceptnode

  real    rhothresh, maxrho, timec

  real   ,allocatable :: rho (:,:,:)
  logical,allocatable :: mask(:,:,:)

  type(treenode),pointer :: proot

  character(2)  mdl
  character(21) dmpfile

  external acceptnode

!---------------------------------------

  inpunit = 1
  dmpunit = 0
  
  open(unit=inpunit,file="coretree.inp",status="unknown")
  read(inpunit,*) logunit
  read(inpunit,*) idebug
  read(inpunit,*) nx
  read(inpunit,*) ny
  read(inpunit,*) nz
  read(inpunit,*) mdl
  read(inpunit,*) dmp
  read(inpunit,*) icore
  read(inpunit,*) maxlevel   ! currently not used.
  read(inpunit,*) rhothresh  ! all points with rhothresh*maxrho < rho < max(rho) are used
  read(inpunit,*) icrit      ! criterion number for acceptnode.
  close(inpunit)

  if (logunit .eq. 2) then 
    open(unit=logunit,file="coretree.log",status="unknown")
  endif
    
  allocate(rho (nx,ny,nz))
  allocate(mask(nx,ny,nz))

  mask(:,:,:) = .true. ! meaning: not occupied

! read the cube
  call corerd(nx, ny, nz, rho, 1, icore, mdl, dmp, timec, logunit)

  maxrho = maxval(rho)
  do k = 1,nz
    do j = 1,ny
      do i = 1,nx
        if (maxrho .eq. rho(i,j,k)) then
          imax = i
          jmax = j
          kmax = k
        endif
      enddo
    enddo
  enddo

  write(logunit,'("CORETREE: max(rho) = ",e13.5," at (",i4,i4,i4,")")') &
       maxrho, imax, jmax, kmax           
  rhothresh = rhothresh*maxrho

! this is not exactly clean... It's the allocation of the root-node.
  call newnode(proot, maxrho, imax, jmax, kmax, 0, null(), 0, logunit, idebug)

! build the tree, starting at proot and maxrho.
  call buildtree(proot, nx, ny, nz, mask, rho, rhothresh, maxrho, acceptnode, icrit, 0, logunit, idebug)
  
  write(logunit,'("CORETREE: building tree finished.")')

! checking whether buildtree did things right.
!  call walktree(proot, logunit, dmpunit, idebug)

!  write(logunit,'("CORETREE: walking tree finished.")')

! deallocate the tree (which can be rather large)
  call deletetree(proot, logunit, idebug)

  write(logunit,'("CORETREE: tree deleted.")')

  count = 0
  do k = 1,nz
    do j = 1,ny
      do i = 1,nx
        if (.not. mask(i,j,k)) then
          count = count + 1
        else
          rho(i,j,k) = 0.0 ! set all non-core points to 0
        endif
      enddo
    enddo
  enddo
  write(logunit,'("CORETREE: ",i8," cells found.")') count

  write(dmpfile,'(a7,i2.2,a3,i3.3,a2,a4)') &
       "subcore",icore,'rho',dmp,mdl,".msk"
  write(logunit,'("CORETREE: writing file ",a21)') dmpfile
  open(file=dmpfile,unit=dmpunit,form="unformatted",status="unknown")
  write(dmpunit) nx,ny,nz,icore,timec
  write(dmpunit) imax,jmax,kmax
  write(dmpunit) rho
  close(dmpunit)

  deallocate(rho)
  deallocate(mask)

  if (logunit .eq. 2) then
    close(logunit)
  endif

!====================================================

end
